@@ -1,6 +1,5 @@
 #' Specific Multiple Correspondence Analysis
 #' @title soc.mca
-#'
 #' \code{soc.mca} performs a specific multiple correspondence analysis on a data.frame of factors, where cases are rows and columns are variables.
 #' @param active       Defines the active modalities in a data.frame with rows of individuals and columns of factors, without NA's'. Active can also be a named list of data.frames. The data.frames will correspond to the analytical headings.
 #' @param sup          Defines the supplementary modalities in a data.frame with rows of individuals and columns of factors, without NA's 
@@ -8,7 +7,7 @@
 #' @param passive      A single character vector with the full or partial names of the passive modalities. All names that have a full or partial match will be set as passive.
 #' @param Moschidis    If TRUE adjusts contribution values for rare modalities. see \link{moschidis}.
 #' @param detailed.results If FALSE the result object is trimmed to reduce its memory footprint.
-#' @param weight a numeric vector with the weights for the individual rows. The weight is normalized afterwardsds.
+#' @param weight a numeric vector with the weights for the individual rows. The weight is normalized afterwards.
 #' 
 #' @return \item{nd}{Number of active dimensions}
 #'  \item{n.ind}{The number of active individuals}
@@ -27,6 +26,7 @@
 #'  \item{coord.ind}{A matrix with the principal coordinates of each individual per dimension.}
 #'  \item{coord.sup}{A matrix with the principal coordinates of each supplementary modality per dimension.}
 #'  \item{names.mod}{The names of the active modalities}
+#'  \item{labels.mod}{The shorter labels of the active modalities}
 #'  \item{names.ind}{The names of the individuals}
 #'  \item{names.sup}{The names of the supplementary modalities}
 #'  \item{names.passive}{The names of the passive modalities}
@@ -89,7 +89,8 @@ soc.mca <- function(active, sup = NULL, identifier = NULL, passive = getOption("
   
   if (data.type == "indicator") {
     a.r <- nrow(active)
-    ind.act <- data.frame(active, check.names = F)
+    #ind.act <- data.frame(active, check.names = F)  #!§ This must be wrong! # Let this comment stay a bit and then remove this line.
+    ind.act <- active                              
     headings = NULL                      # As default, no headings are defined
     }
   
@@ -122,19 +123,22 @@ soc.mca <- function(active, sup = NULL, identifier = NULL, passive = getOption("
   # Creating lists of variable names...
   #########################################
   
-  varlist      <- unique(gsub(": .*", "", colnames(ind.act)))             # Unique varnames
+  varlist      <- unique(gsub(": .*", "", colnames(ind.act)))             # Unique varnames §! This seems weird - but we do it out of necessity - should be remade at some point.
   varlist.long <- gsub(": .*", "", colnames(ind.act))                     # Vector of varnames matching the list of modalities
   Q            <- median(rowSums(ind.act))                                # Number of Questions [we use the median in order to allow for questions that do not sum to one]
   
-  passive.set <- grepl(paste(passive, collapse = "|"), colnames(ind.act)) # Defining the set of passive modalities, default is no passive
+  # passive.set  <- grepl(paste(passive, collapse = "|"), colnames(ind.act)) # Defining the set of passive modalities, default is no passive
+  passive.set  <- purrr::map_lgl(colnames(ind.act), ~ stringr::str_detect(string = .x, pattern = stringr::coll(passive)) %>% any())
+  
   set <- 1:ncol(ind.act)                                                  # set is all
   active.set <- set[!passive.set]                                         # active.set is active modalities
   
   ind.reduced <- ind.act[ ,active.set]                                    # Reducing the indicator matrix to the active set of modalities
   varlist.long.red <- gsub(": .*", "", colnames(ind.reduced))             # The reduced vector of var names, matching the reduced modality name vector
   
-  tmpx <- vector()
   
+# This part here can be written more transparent.  
+  tmpx <- vector()
   #for each variable, 
   for (i in 1:length(unique(varlist))) {
     t1 <- ind.act[,varlist.long == unique(varlist)[i]]
@@ -142,9 +146,12 @@ soc.mca <- function(active, sup = NULL, identifier = NULL, passive = getOption("
     if (is.null(dim(t1)[2])) {
       tmpx[i] <- max(ind.act[,varlist.long == unique(varlist)[i]] - ind.reduced[,varlist.long.red == unique(varlist)[i]])  
     }else{
-      tmpx[i] <- max(rowSums(ind.act[,varlist.long == unique(varlist)[i]]) - rowSums(ind.reduced[,varlist.long.red == unique(varlist)[i]]))
+      tmpx[i] <- max(rowSums(ind.act[,varlist.long == unique(varlist)[i]]) - rowSums(ind.reduced[,varlist.long.red == unique(varlist)[i], drop = FALSE]))
     }
   }
+  
+  
+  
   Qm <- Q - sum(tmpx)                                                     # Calculating the number of questions with passive modalities
   
   
@@ -177,7 +184,7 @@ soc.mca <- function(active, sup = NULL, identifier = NULL, passive = getOption("
   #      The actual analysis, a CA of the indicator matrix       #
   ################################################################
   
-  result <- subset.ca.indicator(ind.act, ind.sup, active.set, passive.set, Q = Q, Qm = Qm, Moschidis = Moschidis, detailed.results = detailed.results, weight = weight)
+  result <- subset_ca_indicator(ind.act, ind.sup, active.set, passive.set, Q = Q, Qm = Qm, Moschidis = Moschidis, detailed.results = detailed.results, weight = weight)
   
   result$variable.all <- varlist.long
   
@@ -188,12 +195,13 @@ soc.mca <- function(active, sup = NULL, identifier = NULL, passive = getOption("
   
   result$names.mod.all            <- colnames(ind.act)
   result$names.mod                <- colnames(ind.act)[active.set]
+  result$labels.mod               <- names(colnames(ind.act))[active.set]
   result$names.ind                <- as.character(identifier)
   result$names.sup                <- colnames(ind.sup)
   result$names.passive            <- colnames(ind.act)[passive.set]
   result$headings.all             <- headings
   result$headings                 <- headings[active.set]
-  result$indicator.matrix.passive <- ind.act[, passive.set]
+  result$indicator.matrix.passive <- ind.act[, passive.set, drop = FALSE]
   result$indicator.matrix.active  <- ind.act[, active.set]
   result$indicator.matrix.all     <- ind.act
   
@@ -331,7 +339,7 @@ soc.mca <- function(active, sup = NULL, identifier = NULL, passive = getOption("
 # ' #@export
 # ' @return a list of various results. See \link{soc.mca} documentation
 
-subset.ca.indicator <- function (ind.act, ind.sup, active.set, passive.set, Q, Qm, 
+subset_ca_indicator <- function (ind.act, ind.sup, active.set, passive.set, Q, Qm, 
                                  Moschidis, detailed.results = FALSE, weight = NULL) {
   
   
@@ -511,7 +519,7 @@ subset.ca.indicator <- function (ind.act, ind.sup, active.set, passive.set, Q, Q
   else {
     freq.sup <- NULL
   }
-  ca.output <- list(nd = R2.dim, n.ind = nrow(Z.act), n.mod = length(active.set), 
+  ca.output <- list(nd = R2.dim, n.ind = nrow(Z.act), n.mod = length(active.set), weight = weight, 
                     eigen = eigen[1:R2.dim], eigen.raw = eigen, Qm = Qm, 
                     Q = Q, total.inertia = sum(eigen[1:R2.dim]), total.inertia.raw = sum(eigen), 
                     adj.inertia = adj.inertia, inertia_full = adj.inertia2, 
@@ -549,35 +557,43 @@ subset.ca.indicator <- function (ind.act, ind.sup, active.set, passive.set, Q, Q
 #' @return Returns a indicator matrix
 #' @seealso \link{soc.mca}
 #' @examples 
-#' a  <- rep(c("A","B"), 5)
+#' a  <- rep(c("A","B"), 5) 
 #' b  <- rep(c("C", "D"), 5)
-#' indicator(data.frame(a,b))
+#' x  <- data.frame(a, b)
+#' indicator(x)
 #' @export indicator
 
 indicator  <- function(x, id = NULL, ps = ": "){
 
-obj         <- x
-I           <- nrow(obj)                                      # Number of individuals
-levels.n    <- unlist(lapply(obj, nlevels))
-n           <- cumsum(levels.n)                               # Number of modalities for each question
-m           <- max(n)                                         # Total number of modalities
-Q           <- ncol(obj)                                      # Number of questions
-Z           <- matrix(0, nrow = I, ncol = m)                  # Predefinition of the indicatormatrix
-newdat      <- lapply(obj, as.numeric)
-offset      <- (c(0, n[-length(n)]))
-for (i in seq(Q)) Z[seq(I) + (I * (offset[i] + newdat[[i]] - 1))] <- 1 # Indicator matrix
-fn          <- rep(names(obj), unlist(lapply(obj, nlevels)))  
-ln          <- unlist(lapply(obj, levels))
-col.names   <- paste(fn, ln, sep = ps)
-colnames(Z) <- col.names
-
-if (identical(id, NULL) == TRUE){
-rownames(Z) <- as.character(seq(I))
-}else{
-rownames(Z) <- id
-}    
-return(Z)
-
+  obj         <- x
+  
+  fct         <- apply(x, 2, is.factor)
+  if(any(fct == FALSE)){
+  obj[, fct == FALSE]         <-   obj[, fct == FALSE, drop = FALSE] |> lapply(as.factor)
+  }
+  
+  I           <- nrow(obj)                                      # Number of individuals
+  levels.n    <- unlist(lapply(obj, nlevels))
+  n           <- cumsum(levels.n)                               # Number of modalities for each question
+  m           <- max(n)                                         # Total number of modalities
+  Q           <- ncol(obj)                                      # Number of questions
+  Z           <- matrix(0, nrow = I, ncol = m)                  # Predefinition of the indicatormatrix
+  newdat      <- lapply(obj, as.numeric)
+  offset      <- (c(0, n[-length(n)]))
+  for (i in seq(Q)) Z[seq(I) + (I * (offset[i] + newdat[[i]] - 1))] <- 1 # Indicator matrix
+  fn          <- rep(names(obj), unlist(lapply(obj, nlevels)))  
+  ln          <- unlist(lapply(obj, levels))
+  col.names   <- paste(fn, ln, sep = ps)
+  colnames(Z) <- col.names
+  names(colnames(Z)) <- ln
+  attr(colnames(Z), "variable") <- fn
+  
+  if (identical(id, NULL) == TRUE){
+    rownames(Z) <- as.character(seq(I))
+  }else{
+    rownames(Z) <- id
+  }    
+  Z
 }
 
 #' Class Specific Multiple Correspondence Analysis
@@ -604,6 +620,7 @@ return(Z)
 #'  \item{coord.sup}{A matrix with the principal coordinates of each supplementary modality per dimension. Notice that the position of the supplementary modalities in class specific analysis is the mean point of the individuals, which is not directly comparable with the cloud of the active modalities.}
 #'  \item{indicator.matrix}{A indicator matrix. See \link{indicator}}
 #'  \item{names.mod}{The names of the active modalities}
+#'  \item{labels.mod}{The shorter labels of the active modalities}
 #'  \item{names.ind}{The names of the individuals}
 #'  \item{names.sup}{The names of the supplementary modalities}
 #'  \item{names.passive}{The names of the passive modalities}
@@ -778,6 +795,7 @@ soc.csa   <- function(object, class.indicator, sup = NULL){
     coord.ind   = csa.i.coord,
     coord.sup   = sup.coord,
     names.mod   = names.mod,
+    labels.mod  = object$labels.mod,
     names.ind   = names.ind,
     names.sup   = names.sup,
     names.passive = names.passive,
@@ -1003,6 +1021,7 @@ supplementary.individuals <- function(object, sup.indicator, replace = FALSE){
 #' Performs tests on what has been passed on to soc.mca by the user.
 #'
 #' @param x the active variables sent to soc.mca
+#' @param check.na If TRUE checks whether x contains NA values and returns an error
 #'
 #' @return a character vector with an evaluation of whether x is data.frame, a list of data.frames, an indicator or a list of indicators.
 #'
@@ -1028,11 +1047,17 @@ supplementary.individuals <- function(object, sup.indicator, replace = FALSE){
 #' 
 #' # Invalid scenarios ----
 #' 
+
+#' # X is a data.frame - but does not contain only factors
+#' x <- taste[, 2:7]
+#' x[, 1] <- x[, 1] |> as.character()
+#' what.is.x(x)  
+
 #' # X is a matrix - but not numeric
 #' x <- as.matrix(taste[, 2:7])
 #' what.is.x(x)  
 #' 
-#' # X is a of data.frames list but does not have names
+#' # X is a list of data.frames list but does not have names
 #' x <- list(taste[, 1:3], taste[, 4:5])
 #' what.is.x(x)
 #' 
@@ -1059,16 +1084,24 @@ supplementary.individuals <- function(object, sup.indicator, replace = FALSE){
 #' what.is.x(x)
 #' }
 
-what.is.x  <- function(x){
+what.is.x  <- function(x, check.na = TRUE){
   # Is it a list of data.frames?
   is.d    <- is.data.frame(x)
   is.l    <- is.list(x)
   is.m    <- is.matrix(x)
   is.lm   <- all(unlist(lapply(x, is.matrix)))
-  is.ld   <- is.d == FALSE & is.l == TRUE & is.lm == FALSE
+  
+  is.mca  <- inherits(x, "soc.mca")
+  is.gpca  <- inherits(x, "gPCA")
+  is.pca  <- inherits(x, "PCA")
+  
+  is.result <- any(is.mca, is.gpca, is.pca)
+  
+  is.ld   <- is.d == FALSE & is.l == TRUE & is.lm == FALSE & is.mca == FALSE
+  is.all.numeric <- all(unlist(lapply(x, is.numeric)))
   
   # Scenario: List
-  if(is.ld | is.lm){
+  if( (is.ld | is.lm) & !is.result){
     not.all.data.frames <- any(!unlist(lapply(x, is.data.frame)))
     not.all.matrix      <- any(!unlist(lapply(x, is.matrix)))
     if(all(not.all.data.frames, not.all.matrix)) stop("x is a list, but not all elements are the same valid type (aka. data.frame or indicator matrix).")
@@ -1080,7 +1113,12 @@ what.is.x  <- function(x){
   if(is.m == TRUE & is.numeric(x) == FALSE) stop("Your data is a matrix, but it is not numeric.") 
   
   # Check for missing
+  if(check.na == TRUE){
   if(anyNA(x, recursive = TRUE)) stop("Your data includes NA. Try recoding to missing.")
+  }
+    
+  # Check for factors
+  if(is.d == TRUE & all(sapply(x, is.factor)) == FALSE) warning("x is a data.frame but not all variables are factors.")
   
   # Is it a list of only matrixes?
   o <- NA
@@ -1089,11 +1127,17 @@ what.is.x  <- function(x){
   if(is.d)  o <- "data.frame"
   # It is surely an indicator matrix
   if(is.m)  o <- "indicator"  
+  if(is.d & is.all.numeric)  o <- "indicator"  
   # It is surely a list of data.frames
   if(is.ld) o <- "list.data.frame"
   # It is surely a list of indicators
   if(is.lm) o <- "list.indicators"
-  
+  # It is an mca  
+  if(is.mca) o <- "soc.mca"
+  # It is an PCA from FactoMineR
+  if(is.pca) o <- "PCA"
+  # It is an gPCA from GDAtools
+  if(is.pca) o <- "gPCA"
   o
 }
 
@@ -1108,3 +1152,44 @@ balance.headings <- function(indicator, headings, Q) {
   }
 }
 
+
+pem_fast<- function(x, y, weights = rep(1, length(x)), digits = 1, sort = FALSE){
+  # This function is a stripped down and faster version of GDAtools::pem()
+  
+  idnona <- !is.na(x) & !is.na(y)
+  X <- x[idnona]
+  Y <- y[idnona]
+  W <- weights[idnona]
+  cont <- xtabs(data = data.frame(X, Y), ~X+Y)
+  
+  tota <- colSums(cont)
+  totb <- rowSums(cont)
+  total <- sum(cont)
+  theo <- matrix(nrow = nrow(cont), ncol = ncol(cont))
+  for (i in 1:nrow(cont)) {
+    for (j in 1:ncol(cont)) theo[i, j] <- tota[j] * totb[i]/total
+  }
+  ecart <- cont - theo
+  max <- matrix(nrow = nrow(cont), ncol = ncol(cont))
+  emax <- matrix(nrow = nrow(cont), ncol = ncol(cont))
+  pem <- matrix(nrow = nrow(cont), ncol = ncol(cont))
+  for (i in 1:nrow(cont)) {
+    for (j in 1:ncol(cont)) {
+      if (ecart[i, j] >= 0) 
+        max[i, j] <- min(tota[j], totb[i])
+      if (ecart[i, j] < 0 & tota[j] <= (total - totb[i])) 
+        max[i, j] <- 0
+      if (ecart[i, j] < 0 & tota[j] > (total - totb[i])) 
+        max[i, j] <- tota[j] + totb[i] - total
+      emax[i, j] <- max[i, j] - theo[i, j]
+      pem[i, j] <- ifelse(ecart[i, j] >= 0, ecart[i, j]/emax[i, 
+                                                             j] * 100, 0 - ecart[i, j]/emax[i, j] * 100)
+    }
+  }
+  dimnames(pem) <- dimnames(cont)
+  
+  pem <- as.table(pem)
+  rownames(pem) <- gsub("data.", "", rownames(pem))
+  colnames(pem) <- gsub("data.", "", colnames(pem))
+  round(as.table(pem), digits)
+}
