@@ -479,6 +479,105 @@ average.coord <- function(object, x, dim = c(1, 2)){
   point.coord
 }
 
+#' Supplementary coordinates for a data.frame of factors
+#'
+#' Calculate the average coordinates in the category cloud of a soc.mca analysis.
+#' 
+#' @param object a soc.mca result object
+#' @param sup a data.frame of factors or an indicator matrix
+#' @param dim a numeric vector with the two dimensions calculated
+#'
+#' @return a data.frame with coordinates and labels
+#' @export
+#'
+#' @examples
+#' example(soc.mca)
+#' supplementary.categories(result, sup)
+supplementary.categories <- function(object, sup, dim = 1:2){
+  
+  sup.is    <- what.is.x(sup, check.na = FALSE)
+  object.is <- what.is.x(object)
+  
+  
+  # If sup is an indicator
+  if(sup.is == "indicator"){
+    
+    coord <- object$coord.ind[, dim]
+    ind.x <- sup * coord[, 1] 
+    ind.y <- sup * coord[, 2]
+    ind.x[ind.x == 0] <- NA
+    ind.y[ind.y == 0] <- NA
+    
+    cmx   <- colMeans(ind.x, na.rm = TRUE) / sqrt(object$eigen[dim[1]])  
+    cmy   <- colMeans(ind.y, na.rm = TRUE) / sqrt(object$eigen[dim[2]])
+    
+    # Variable names
+    
+    var.names <- attr(colnames(sup), "variable")
+    if(length(var.names) != ncol(sup)) var.names <- NULL
+    if(is.null(var.names)) var.names <- colnames(sup) %>% strsplit(": ") %>% map_chr(~head(.x, 1))
+    
+    labels <- names(colnames(sup))
+    if(is.null(labels)) labels <- colnames(sup) %>% strsplit(": ") %>% map_chr(~tail(.x, 1))
+    
+    out <- tibble(Variable = var.names, label = labels, X = cmx, Y = cmy, Frequency = colSums(sup), Modality = colnames(sup))
+    return(out)
+  }
+  
+  sup <- sup %>% tibble()
+  # Add tests
+  out   <- map(sup, ~average.coord(object, .x, dim)) %>% bind_rows(.id = "Variable")
+  out   <- out %>% select(Variable, label = label, X, Y, Frequency = Freq) %>% tibble() %>% mutate(Modality = paste0(Variable, ": ", label))
+  out
+}
+
+
+supplementary_variable <- function(res, var) {
+  
+  eigen    <- res$eigen
+  if(length(eigen) != ncol(res$coord.ind)){
+  eigen    <- res$eigen[1:ncol(res$coord.ind)]
+  warning("Your soc.ca object has been trimmed prior to this function. This may influence calculations")
+  }
+  
+  weight   <- res$weight
+  variable <- factor(var)
+  n  <- sum(weight)
+  
+  FK <- colSums(weight * (indicator(as.data.frame(variable))))/n
+  ind <- res$coord.ind
+  coord <- aggregate(weight * ind, list(variable), sum)[, -1]/n/FK
+  vrc   <- aggregate(weight * ind * ind, list(variable), sum)[, -1]/n/FK - coord * coord
+  
+  for (i in 1:ncol(coord)) coord[, i] <- coord[, i]/ sqrt(eigen[i])
+  cos2 <- coord * coord/((1/FK) - 1)
+  freq = n * FK
+  
+  s <- 1:length(eigen)
+  
+  names(freq) <- levels(variable)
+  rownames(coord) <- levels(variable)
+  rownames(cos2) <- levels(variable)
+  wi <- apply(vrc, 2, weighted.mean, w = freq)
+  be <- eigen - wi[s]
+  eta2 <- be/eigen[s]
+  
+  vrc <- rbind(vrc[s], wi[s], be, eigen[1:length(be)], eta2)
+  vrc <- round(vrc, 6)
+  
+  rownames(vrc) <- c(levels(variable), "within", "between", "total", 
+                     "eta2")
+  coord <- round(coord, 6)
+  typic <- sqrt(cos2) * sqrt(length(variable) - 1)
+  typic <- (((abs(coord) + coord)/coord) - 1) * typic
+  pval <- 2 * (1 - pnorm(abs(as.matrix(typic))))
+  list(freq = round(freq, 1), coord = coord, cos2 = round(cos2, 
+                                                          6), var = round(vrc, 6), typic = round(typic, 6), pval = round(pval, 
+                                                                                                                         6))
+}
+
+
+
 #' CSA measures
 #' 
 #' Several measures for the evaluation of the relations between the dimensions of the CSA and the dimensions the of original MCA
@@ -644,8 +743,10 @@ headings      <- function(object, dim = 1:3) {
   
   head.ctr.total <- rep(1, (max(dim) + 3))
   
-  tot    <- aggregate(rowSums(object$ctr.mod.raw), by = list(object$headings), sum)
+  tot     <- tibble(ctr = rowSums(object$ctr.mod.raw), heading = object$headings %>% fct() %>% fct_relevel(lev.head)) %>% group_by(heading) %>% summarise(x = sum(ctr))
   tot$x  <- round(tot$x / sum(tot$x) * 100,1)
+  # tot    <- aggregate(rowSums(object$ctr.mod.raw), by = list(object$headings), sum)
+  # tot$x  <- round(tot$x / sum(tot$x) * 100,1)
   
   for (i in seq(length(lev.head))) {
     var.under.head     <- head.var[which(head.var[,1] == lev.head[i]),2]
